/*
 * @(#)WelltrisP.h
 *
 * Copyright 1993 - 2008  David A. Bagley, bagleyd@tux.org
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995  Q. Alex Zhao, azhao@cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Private header file for Welltris */

#ifndef _WelltrisP_h
#define _WelltrisP_h

#include "xwin.h"
#include "timer.h"
#include "polyomino.h"
#include "Welltris.h"

#ifdef WINVER
extern void destroyGame(HBRUSH brush);
extern void resizeGame(WelltrisWidget w);
extern void sizeGame(WelltrisWidget w);
extern void initializeGame(WelltrisWidget w, HBRUSH brush);
extern void exposeGame(WelltrisWidget w);
extern void hideGame(WelltrisWidget w);
extern void clearGame(WelltrisWidget w);
extern void enterGame(WelltrisWidget w);
extern void leaveGame(WelltrisWidget w);
extern void moveGameInput(WelltrisWidget w, int direction);
extern void soundGame(WelltrisWidget w);
extern Boolean incrementGameLevel(WelltrisWidget w);
extern Boolean decrementGameLevel(WelltrisWidget w);
extern Boolean incrementGameUnits(WelltrisWidget w);
extern Boolean decrementGameUnits(WelltrisWidget w);
extern Boolean incrementGameDiameter(WelltrisWidget w);
extern Boolean decrementGameDiameter(WelltrisWidget w);
extern void fixedfillGame(WelltrisWidget w);
extern void suddenAppearGame(WelltrisWidget w);
extern Boolean diagonalGame(WelltrisWidget w);
extern Boolean mixedGame(WelltrisWidget w);
extern void bonusGame(WelltrisWidget w);
extern void cwGame(WelltrisWidget w);
extern void plainPieceGame(WelltrisWidget w);
extern void falldownGame(WelltrisWidget w);
extern void pauseGame(WelltrisWidget w);
extern void showNextGame(WelltrisWidget w);
extern void gridGame(WelltrisWidget w);
extern void fillGame(WelltrisWidget w, int n);
extern void setGame(WelltrisWidget w, int reason);
extern void setGameNext(WelltrisWidget w, int reason,
	int randomNumber, int randomRotation, int randomReflection,
	int units, Boolean diagonal, Boolean mixed, Boolean newBonus);

#else

/* This gets around C's inability to do inheritance */
typedef struct _WelltrisClassPart {
	int         ignore;
} WelltrisClassPart;

typedef struct _WelltrisClassRec {
	CoreClassPart coreClass;
	WelltrisClassPart welltrisClass;
} WelltrisClassRec;

extern WelltrisClassRec welltrisClassRec;
#endif

#ifndef BUMPSOUND
#define BUMPSOUND "bump"
#endif
#ifndef DRIPSOUND
#define DRIPSOUND "drip"
#endif

#define SYMBOL ':'

#define NUM_FLASHES 4

#define MAX_SIDES 4
#define MIN_LEVELS 1
#define MAX_LEVELS 20
#define FG_SHADES 5
#define BG_SHADES 5

#define THRESHOLD(x)	((x)*10)
#define EMPTY		(-1)
#define DRAW_COLOR	0
#define XOR_COLOR	(-1)
#define XOR_CLEAR_COLOR	(-2)
#define CLEAR_COLOR	(-3)
#define BORDER_COLOR	(-4)
/* OLD
#define MAX_WIDTH	8
#define MAX_DEPTH	12
#define MAX_SIDES	4
#define MAX_PERIMETER	(MAX_SIDES*MAX_WIDTH)

#define	BOXSIZE		18
#define	NEXTBOXSIZE	16
#define DELTA		1
#define BASE_WIDTH	(MAX_WIDTH*BOXSIZE+DELTA)
#define ENDPT		(3*BASE_WIDTH)
#define ACTION_WIDTH	(ENDPT+2*DELTA)
#define MIDPT		(ENDPT/2+DELTA)
#define	OFFSET		20
*/

/*** types ***/

typedef enum {FALL, DROP, MOVE_LEFT, MOVE_RIGHT, ROTATE, REFLECT} move_t;

typedef struct {
	int         units;
	Boolean     diagonal, mixed, bonus;
} Spec;

typedef struct {
	int         units, polyNumber;
	int         xpos, ypos;
	int         size, colorNumber;
	int         bonus, randomRotation, randomReflection;
	long        randomNumber;
} Thing;

typedef struct {
	int         pmid, cid;
} Field;

typedef struct _WelltrisPart {
	Thing       curThing, nextThing;
	Field      *wall, *base;
	GC          thingGC[MAX_START_POLYOMINOES][FG_SHADES];
	Boolean     fixedFill, suddenAppear, cw, plainPiece, showNext, grid;
	Boolean     sound, resetTime, firstFall, paused, gotNew;
	int         level, fill, completedRows;
	int         startLevel, score, newRows;
	Spec        current, next;
	int         units;
	int         frozenWall[MAX_SIDES];
	Boolean     diagonal, mixed, bonus;
	int         delay, numberPolyominoes, pauseStart;
	int         currentPosition, dropPaused;
	Boolean     fall, started, quit, act;
	Boolean     mono, reverse, focus;
	Boolean     scoreOnly, versionOnly;
	int         boxSize;
	Position    viewLength;
	Point       offset;
	GC          xorGC;
	GC          inverseGC[BG_SHADES];
	int         diameter, depth, perimeter, endPt;
#ifdef WINVER
	char        userName[81], scoreFile[81], bumpSound[81], dripSound[81];
#else
	int         menu, pixmapSize;
	char       *userName, *scoreFile, *bumpSound, *dripSound;
	Pixel       foreground, background;
	Pixel       thingColor[MAX_START_POLYOMINOES];
	XtCallbackList select;
#endif
} WelltrisPart;

typedef struct _WelltrisRec {
	CorePart    core;
	WelltrisPart  welltris;
} WelltrisRec;

#ifdef WINVER
extern int brighter(const int light);
extern int darker(const int light);
#else
extern Pixel brighter(WelltrisWidget w, Pixel pixel);
extern Pixel darker(WelltrisWidget w, Pixel pixel);
#endif
extern void redoNext(WelltrisWidget w);
#endif /* _WelltrisP_h */
